@@ -1,5 +1,5 @@
 /*-*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: IntegerObject.java,v 1.1 2000/11/12 14:59:34 metlov Exp $
+ * $Id: IntegerObject.java,v 1.2 2001/02/15 00:07:50 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -29,5 +29,9 @@ public class IntegerObject implements gnu.jel.reflect.Integer {
     return 1;
   };
 
+  public String getStringProperty(String name) {
+    return name;
+  };
+
 };
 
