@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: TestSuite.java,v 1.32 2001/06/08 15:48:59 metlov Exp $
+ * $Id: TestSuite.java,v 1.35 2001/11/07 22:37:07 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -42,7 +42,7 @@ public class TestSuite {
       System.out.println(e);
       System.exit(1);
     };
-    Library lib=new Library(staticLib,null,dotAllowedOn);
+    Library lib=new Library(staticLib,null,dotAllowedOn,null,null);
     try {
       lib.markStateDependent("random",null);
     } catch (CompilationException e) {
@@ -296,7 +296,7 @@ public class TestSuite {
     vp.xvar=5.0;
     rtp[0]=vp;
     dynamicLib[0]=vp.getClass();
-    lib=new Library(staticLib,dynamicLib);
+    lib=new Library(staticLib,dynamicLib,null,null,null);
     
     testExpression("sin(x/5)",new Double(Math.sin(1.0)),null,rtp,lib,o);
     testExpression("255+5+7+9-x",new Double(255+7+9),null,rtp,lib,o);
@@ -351,7 +351,7 @@ public class TestSuite {
       System.exit(1);
     };
     
-    lib=new Library(staticLib,dynamicLib,dotLib);
+    lib=new Library(staticLib,dynamicLib,dotLib,null,null);
 
     testExpression("(\"abc\").length()", new Integer(3), null, rtp, lib,o);
     testExpression("\"abc\".length()", new Integer(3), null, rtp, lib,o);
@@ -379,7 +379,7 @@ public class TestSuite {
 
 
     o.println(); o.println("----- DYNAMIC VARIABLES ------");
-    lib=new Library(staticLib,dynamicLib,dotLib,vp);
+    lib=new Library(staticLib,dynamicLib,dotLib,vp,null);
     vp.addProperty("p1","p1value");
     vp.addProperty("p1.s1","p1s1value");
     vp.addProperty("p1.s2","p1s2value");
@@ -461,6 +461,16 @@ public class TestSuite {
     testExpression("anObject!=anObject",Boolean.FALSE,null, rtp, lib,o);
     testExpression("!(anObject==anObject)",Boolean.FALSE,null, rtp, lib,o);
 
+    vp.addProperty("\u3050\u3051","Hiragana");
+    testExpression("\u3050\u3051+\"-works\"", "Hiragana-works",
+                   null, rtp, lib,o);
+    vp.addProperty("\u3106\u3107","Bopomofo");
+    testExpression("\u3106\u3107+\"-works\"", "Bopomofo-works",
+                   null, rtp, lib,o);
+
+    testExpression("_T_g", "_U_g",null, rtp, lib,o);
+    testExpression("_T_j", "_U_j",null, rtp, lib,o);
+    testExpression("_T_j+_T_g", "_U_j_U_g",null, rtp, lib,o);
 
     o.println(); o.println("----- Automatic widening conversions of "+
                            "references ------");
@@ -555,7 +565,7 @@ public class TestSuite {
         newDynamicLib[newDynamicLib.length-1]=
           newRTP[newRTP.length-1].getClass();
         
-        Library lib1=new Library(staticLib,newDynamicLib,new Class[0],vp);
+        Library lib1=new Library(staticLib,newDynamicLib,new Class[0],vp,null);
         testExpression("aMethod", new Integer(1),null,newRTP,lib1,o);
       };
     };
@@ -1079,11 +1089,13 @@ class OpenEvaluator extends Evaluator {
       fos.write(image);
       fos.close();
     } catch (java.io.IOException exc) {
-      gnu.jel.debug.Debug.reportThrowable(exc);
+      System.out.println("Error writing \"dump.class\"");
+      System.out.println(exc);
     };
 	try {
       return (CompiledExpression)(ImageLoader.load(image)).newInstance();
 	} catch (Exception exc) {
+      exc.printStackTrace();
       return null;
 	};
   };
