@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: Evaluator.java,v 1.11 2000/11/12 14:59:34 metlov Exp $
+ * $Id: Evaluator.java,v 1.14 2001/08/23 14:30:21 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -15,12 +15,9 @@
 package gnu.jel; 
 
 import gnu.jel.generated.EC;
+import gnu.jel.generated.CharStream;
 import gnu.jel.debug.Debug;
-import gnu.jel.reflect.Field;
-import gnu.jel.reflect.LocalField;
-import gnu.jel.reflect.Method;
-import gnu.jel.reflect.LocalMethod;
-import gnu.jel.reflect.Member;
+import java.lang.reflect.Member;
 
 /**
  * This is the main front end to JEL.
@@ -83,7 +80,7 @@ public class Evaluator {
       cf.newMethod(cnstr,null);
       cf.code(0x2a);                //| aload_0  ;loads "this"
       cf.typesStk.pushID(11); // not important what, it must be a reference
-      Method supInit=new Method(cmplExpr.getConstructor(new Class[0]));
+      Member supInit=cmplExpr.getConstructor(new Class[0]);
       cf.code(0xb7);                //| invokespecial
       cf.codeI(cf.getIndex(supInit,10));  //|    super();
       cf.typesStk.pop();
@@ -189,8 +186,7 @@ public class Evaluator {
 
   protected static OPlist parse(String expression, Library lib,
                                 Class resultType) throws CompilationException {
-    return 
-      (new EC(new java.io.StringReader(expression))).parse(resultType,lib);
+    return (new EC(new CharStream(expression))).parse(resultType,lib);
   };
 
   protected static byte[] getImage(OPlist list) {
@@ -199,7 +195,7 @@ public class Evaluator {
     ClassFile cf=(ClassFile)cf_orig.clone();
 
     // set return type
-    cf.textData.patchAddress(retID_patchback,(byte)retID);
+    cf.textData.patch(retID_patchback,(byte)retID);
 
     // add the evaluate method
     cf.newMethod(eval_methods[retID],null);
