@@ -1,5 +1,5 @@
 /*-*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: Library.java,v 1.12 2001/03/07 20:19:23 metlov Exp $
+ * $Id: Library.java,v 1.15 2001/11/07 22:38:21 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -15,16 +15,17 @@
 package gnu.jel;
 
 import gnu.jel.debug.Debug;
-import gnu.jel.debug.Tester;
-import gnu.jel.reflect.Field;
-import gnu.jel.reflect.LocalField;
-import gnu.jel.reflect.Method;
-import gnu.jel.reflect.LocalMethod;
-import gnu.jel.reflect.Member;
+import java.lang.reflect.Member;
+import java.lang.reflect.Field;
+import java.lang.reflect.Method;
 import java.util.Hashtable;
 import java.util.Vector;
 import java.util.Enumeration;
 
+//+-+-+-+TESTS
+//-import gnu.jel.debug.Tester;
+//+-+-+-+TESTS
+
 /**
  * A namespace for JEL expressions.
  * <P> There are two types of members in the library, those which are stateless
@@ -70,29 +71,56 @@ public class Library {
    *  methods are exported.
    * @param dynamicLib is the array of classes, whose public virutal
    *  methods are exported.
+   * @deprecated Please us 5 argument constructor with unused arguments
+   *  set to <TT>null</TT>. This constructor is scheduled for removal in
+   *  JEL 1.0.
    */
   public Library(Class[] staticLib,Class[] dynamicLib) {
     this(staticLib,dynamicLib,null);
   };
 
+  /**
+   * Creates a library for JEL.
+   * <P> See the three argument constructor for more info.
+   * @param staticLib is the array of classes, whose public static 
+   *  methods are exported.
+   * @param dynamicLib is the array of classes, whose public virutal
+   *  methods are exported.
+   * @param dotClasses Controls access to the "dot" operator on classes.
+   * @deprecated Please us 5 argument constructor with unused arguments
+   *  set to <TT>null</TT>. This constructor is scheduled for removal in
+   *  JEL 1.0.
+   */
   public Library(Class[] staticLib,Class[] dynamicLib,Class[] dotClasses) {
     this(staticLib,dynamicLib,dotClasses,null);
   };
 
+  /**
+   * Creates a library for JEL.
+   * <P> See the three argument constructor for more info.
+   * @param staticLib is the array of classes, whose public static 
+   *  methods are exported.
+   * @param dynamicLib is the array of classes, whose public virutal
+   *  methods are exported.
+   * @param dotClasses Controls access to the "dot" operator on classes.
+   * @param resolver Controls resolution of dynamic variables.
+   * @deprecated Please us 5 argument constructor with unused arguments
+   *  set to <TT>null</TT>. This constructor is scheduled for removal in
+   *  JEL 1.0.
+   */
   public Library(Class[] staticLib,Class[] dynamicLib,Class[] dotClasses,
                  DVResolver resolver) {
     this(staticLib,dynamicLib,dotClasses,resolver,null);
   };
 
-
   /**
    * Creates a library for JEL.
    * <P> The following should be kept in mind when constructing a library:
    * <OL>
    * <LI>This constructor may throw IllegalArgumentException if it does not
    * like something in Your library. The requirements to the method names
-   * are somewhat more strict, than in Java as the <TT>class.method(..)</TT>
-   * syntax is not used.
+   * are somewhat more strict, than in Java because members of several
+   * classes can be merged in root namespace.
    * <LI>When calling the 
    * <TT>CompiledExpression.evaluate(Object[] dynalib)</TT> of the
    * expression, using dynamic library methods it is needed to pass as
@@ -116,13 +144,13 @@ public class Library {
    * of the form <object>.(<method>|<field>), 
    * which means to call method (access field)
    * of an <object>. There can be three types of the behaviour: 
-   * <BR>1) dot operators are prohibited (<TT>dotClasses==null</TT>),
+   * <P>1) dot operators are prohibited (<TT>dotClasses==null</TT>),
    * this is behaviour of older version of JEL.
-   * <BR>2) dot operators are allowed on all classes
+   * <P>2) dot operators are allowed on all classes
    * (<TT>dotClasses==new Class[0], an empty array).
    * Depending on the types of objects returned by the static/dynamic library
    * classes this may pose a security risk.
-   * <BR>3) dot operators are allowed only on some classes. This is achieved 
+   * <P>3) dot operators are allowed only on some classes. This is achieved 
    * by listing these classes in the dotClasses array.
    * @param staticLib is the array of classes, whose public static 
    *  methods are exported.
@@ -131,6 +159,7 @@ public class Library {
    * @param dotClasses is the array of classes on which the dot ('.')
    * operation is allowed.
    * @param resolver is the object used to resolve the names.
+   * @param cnmap Maps class names into classes for non-primitive type casts.
    */
   public Library(Class[] staticLib,Class[] dynamicLib,Class[] dotClasses,
                  DVResolver resolver,Hashtable cnmap) {
@@ -144,7 +173,7 @@ public class Library {
       // hash the names
       Class[] temp=new Class[1];
       for(int i=0;i<dotClasses.length;i++) 
-        rehashDotClass(dotClasses[i]);
+        rehash(dotClasses[i]);
     };
 
     names = new Hashtable();
@@ -157,7 +186,7 @@ public class Library {
       rehash(dynamicLib,names,dynIDs,null);
   };
 
-  private void rehashDotClass(Class cls) {
+  private void rehash(Class cls) {
     Hashtable tempNames=new Hashtable();
     Class[] temp=new Class[1];
     temp[0]=cls;
@@ -171,15 +200,13 @@ public class Library {
     for (int i=0; i<arr.length; i++) {
       Integer dynID=new Integer(i);
 
-      java.lang.reflect.Method[] marr=arr[i].getMethods();
-      java.lang.reflect.Field[] farr=arr[i].getFields();
+      Method[] marr=arr[i].getMethods();
+      Field[] farr=arr[i].getFields();
 
       int totalMethods=marr.length;
       int totalMembers=totalMethods+farr.length;
       for(int j=0;j<totalMembers;j++) {
-        Member m = j<totalMethods?
-          (Member)new Method(marr[j]):
-          (Member)new Field(farr[j-totalMethods]);
+        Member m = j<totalMethods?(Member)marr[j]:(Member)farr[j-totalMethods];
         if ((m.getModifiers() & 0x0008)>0) { // static
           if ((stateless!=null) && rehash(hashedNames,m)) 
             stateless.put(m,Boolean.TRUE);
@@ -193,11 +220,12 @@ public class Library {
 
   private static boolean rehash(Hashtable hashedNames, Member m) {
     String name=m.getName();
-    String signature=ClassFile.getSignature(m);
+    String signature=TypesStack.getSignature(m);
 
     // for the purpose of matching fields behave like methods with no
     // arguments
-    if (m instanceof Field) signature="()"+signature;
+    if ((m instanceof Field) || (m instanceof LocalField))
+      signature="()"+signature;
     
     Hashtable signatures=(Hashtable)hashedNames.get(name);
     if (signatures==null) { 
@@ -236,7 +264,7 @@ public class Library {
    */
   public void markStateDependent(String name, Class[] params) 
        throws CompilationException {
-    Object m=getMember(name,params);
+    Object m=getMember(null,name,params);
     Object removed=stateless.remove(m);
     if (Debug.enabled)
       Debug.assert(removed!=null,"State dependent methos \""+m+
@@ -253,25 +281,12 @@ public class Library {
     return stateless.containsKey(o);
   };
 
-  public Class getClassByName(String name) {
-    if (cnmap==null) return null;
-
-    if (Debug.enabled) {
-      // Just to print a meaningful diagnostics in the case user puts
-      // non-class object into the cnmap
-      Debug.assert(cnmap.get(name) instanceof Class,"All elements of cnmap "+
-                   "must be classes and \""+name+"\" is not.");
-    };
-
-    return (Class)cnmap.get(name);
-  };
-  
   /**
-   * Used to search library for method or field.
+   * Searches the namespace defined by this library object for method or field.
    * <P> The method with the same name, and closest (convertible) parameter
    * types is returned. If there are several methods the most specific one
    * is used. 
-   * Ambiguities are detected. Example of detectable ambiguity:<BR>
+   * <P>Ambiguities are detected. Example of detectable ambiguity:<BR>
    * You want to call : <TT>someName(int, int)<TT> <BR>
    * There are two applicable methods : <TT>someName(int, double)<TT>,
    * <TT>someName(double, int)<TT> , requirements to parameter types of each
@@ -283,22 +298,8 @@ public class Library {
    * methods in the same class. However, as this library is assembled from
    * several Java classes such ambiguities can happen, and should be
    * detected.
-   * @param name is the name of the method to find.
-   * @param params are the types of formal parameters in the method invocation.
-   * @return the method/field object of the resolved method/field.
-   * @exception CompilationException if the method can't be resolved
-   */
-  public Member getMember(String name,Class[] params) 
-    throws CompilationException {
-    return getMember(names,null,name,params);
-  };
-
-  /**
-   * Finds the member of a given class by name.
-   * <P> This method is used in compiling the dot ('.') operators.
-   * It does all necessary security checks and wraps the result
-   * into one of JEL's reflection classes.
-   * @param container the class to search the method within.
+   * @param container the class to search the method within, if <TT>null</TT>
+   *                  the root namespace is searched.
    * @param name is the name of the method to find.
    * @param params are the types of formal parameters in the method invocation.
    * @return the method/field object of the resolved method/field.
@@ -306,52 +307,39 @@ public class Library {
    */
   public Member getMember(Class container,String name,Class[] params) 
     throws CompilationException {
-    // security
-    if (dotClasses==null) // dot operator is prohibited
-      throw new CompilationException(-1,11,null);
-    else if (! (noDotSecurity || dotClasses.containsKey(container))) {
-      // dot is not allowed in this particular class
-      Object[] paramsExc={container};
-      throw new CompilationException(-1,12,paramsExc);
-    };
-    Hashtable hashedMembers=(Hashtable)dotClasses.get(container);
-    if (hashedMembers==null) {
-      rehashDotClass(container);
-      hashedMembers=(Hashtable)dotClasses.get(container);
-    };
-      
-    return getMember(hashedMembers,container,name,params);;
-  };
+    Hashtable hashedMembers=names;
 
+    // dot operator security    
+    if (container!=null) {
+      if (dotClasses==null) // dot operator is prohibited
+        throw new CompilationException(-1,11,null);
+      else if (! (noDotSecurity || dotClasses.containsKey(container))) {
+        // dot is not allowed in this particular class
+        Object[] paramsExc={container};
+        throw new CompilationException(-1,12,paramsExc);
+      };
+      if ((hashedMembers=(Hashtable)dotClasses.get(container))==null) {
+        rehash(container);
+        hashedMembers=(Hashtable)dotClasses.get(container);
+      };
+    };
+    
 
-  // This matches a method to a given invocation pattern,
-  // unlike Class.getMethod(...) tries various widening conversions.
-  private static Member getMember(Hashtable hashedMembers, Class cls,
-                                  String name,Class[] params) 
-    throws CompilationException {
     Hashtable signatures=(Hashtable) hashedMembers.get(name);
 
-//      System.out.print("RESOLVING: ");
-//      if(cls!=null) System.out.print(""+cls+".");
-//      System.out.print(name+"(");
-//      if (params!=null) {
-//        for(int i=0;i<params.length;i++) {
-//          System.out.print(params[i]);
-//          if (i<params.length-1) System.out.print('.');
-//        };
-//      };
-//      System.out.println(")");
+    //      System.out.print("RESOLVING: ");
+    //      System.out.println(describe(name,params));
 
     if (signatures==null) { // name is not found
-      Object[] paramsExc={name,cls};
-      throw new CompilationException(-1,cls==null?5:6,paramsExc);
+      Object[] paramsExc={name,container};
+      throw new CompilationException(-1,container==null?5:6,paramsExc);
     };
     
     // Choose applicable methods
     Vector applicable_methods=new Vector();
     for(Enumeration e=signatures.elements();e.hasMoreElements();) {
       Member cm = (Member)e.nextElement();
-      Class[] cp=cm.getParameterTypes();
+      Class[] cp=TypesStack.getParameterTypes(cm);
       
       boolean applicable=false;
       if (params!=null) { // if no parameters requested
@@ -369,8 +357,8 @@ public class Library {
     };
     
     if (applicable_methods.size()==0) {
-      Object[] paramsExc={name,describe(name,params),cls};
-      throw new CompilationException(-1,cls==null?7:8,paramsExc);
+      Object[] paramsExc={name,describe(name,params),container};
+      throw new CompilationException(-1,container==null?7:8,paramsExc);
     };
     
     if (applicable_methods.size()==1) 
@@ -379,14 +367,14 @@ public class Library {
     // Search for the most specific method
     Enumeration e=applicable_methods.elements();
     Member most_specific=(Member)e.nextElement();
-    Class[] most_specific_params=most_specific.getParameterTypes();
+    Class[] most_specific_params=TypesStack.getParameterTypes(most_specific);
 
     //    System.out.println("--- APPLICABLE METHODS ---");
     //    System.out.println(most_specific.getName()+
     //                       ClassFile.getSignature(most_specific));
     while (e.hasMoreElements()) {
       Member cm= (Member)e.nextElement();
-      Class[] cp=cm.getParameterTypes();
+      Class[] cp=TypesStack.getParameterTypes(cm);
       boolean moreSpecific=true;
       boolean lessSpecific=true;
 
@@ -408,8 +396,8 @@ public class Library {
         Object[] paramsExc={describe(name,most_specific_params),
                            describe(name,cp),
                            describe(name,params),
-                           cls};
-        throw new CompilationException(-1,cls==null?9:10,paramsExc);        
+                           container};
+        throw new CompilationException(-1,container==null?9:10,paramsExc);        
       };
     };
     //    System.out.println("--- END APPLICABLE METHODS ---");
@@ -417,19 +405,17 @@ public class Library {
   };
 
   protected static String describe(String name,Class[] params) {
-    String res="";
     StringBuffer invp=new StringBuffer();
     invp.append(name); 
     invp.append('(');
     
     if (params!=null)
       for(int k=0;k<params.length;k++) {
-	if (k!=0) invp.append(',');
-	invp.append(params[k].toString());
+        if (k!=0) invp.append(',');
+        invp.append(params[k].toString());
       };
     invp.append(')');
-    res=invp.toString();
-    return res;
+    return invp.toString();
   };
   
   /**
@@ -447,240 +433,240 @@ public class Library {
     return id.intValue();
   };
   
-  
-  // ------------------------- TESTSUITE -------------------------------
-  
-  /**
-   * Performs unitary test of the library.
-   * @param args ignored.
-   */
-  public static void main(String[] args) {
-    Tester t=new Tester(System.out);
-    test(t);
-    t.summarize();
-  };
-
-  /**
-   * Performs unitary test of the library.
-   * <p> Used if all package is being tested and not just codegen.
-   * @param t Tester to report test results.
-   */
-  public static void test(Tester t) {
-    Library ll=null;
-    Class math=null;
-    try {
-      math=Class.forName("java.lang.Math");
-    } catch(ClassNotFoundException e) {
-      Debug.println("It is IMPOSSIBLE :)");
-    };
-
-    t.startTest("Creating the library of java.lang.Math");
-    try {
-      Class[] sl=new Class[1];
-      sl[0]=math;
-      Library l=new Library(sl,null);
-      ll=l;
-      t.testOK();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Attempt of invocation round(double)");
-    try {
-      Class[] par=new Class[1];
-      par[0]=Double.TYPE;
-      Member mf=ll.getMember("round",par);
-      if ((mf!=null) && 
-          (mf.equals(new Method(math.getMethod("round",par)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Attempt of invocation round(float)");
-    try {
-      Class[] par=new Class[1];
-      par[0]=Float.TYPE;
-      Member mf=ll.getMember("round",par);
-      if ((mf!=null) && 
-          (mf.equals(new Method(math.getMethod("round",par)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Attempt of invocation round(int) best is round(float)");
-    try {
-      Class[] par=new Class[1];
-      par[0]=Integer.TYPE;
-      Member mf=ll.getMember("round",par);
-
-      Class[] par1=new Class[1];
-      par1[0]=Float.TYPE;
-
-
-      if ((mf!=null) && 
-          (mf.equals(new Method(math.getMethod("round",par1)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Attempt of invocation abs(int) best is abs(int)");
-    try {
-      Class[] par=new Class[1];
-      par[0]=Integer.TYPE;
-      Member mf=ll.getMember("abs",par);
-
-      Class[] par1=new Class[1];
-      par1[0]=Integer.TYPE;
-
-      if ((mf!=null) && 
-          (mf.equals(new Method(math.getMethod("abs",par1)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Attempt of invocation abs(byte) best is abs(int)");
-    try {
-      Class[] par=new Class[1];
-      par[0]=Byte.TYPE;
-      Member mf=ll.getMember("abs",par);
-      
-      Class[] par1=new Class[1];
-      par1[0]=Integer.TYPE;
-      
-      if ((mf!=null) && 
-          (mf.equals(new Method(math.getMethod("abs",par1)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Attempt of invocation abs(char) best is abs(int)");
-    try {
-      Class[] par=new Class[1];
-      par[0]=Character.TYPE;
-      Member mf=ll.getMember("abs",par);
-      
-      Class[] par1=new Class[1];
-      par1[0]=Integer.TYPE;
-      
-      if ((mf!=null) && 
-          (mf.equals(new Method(math.getMethod("abs",par1)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-    
-    t.startTest("Attempt of invocation min(int,float) best is min(float,float)");
-    try {
-      Class[] par=new Class[2];
-      par[0]=Integer.TYPE;
-      par[1]=Float.TYPE;
-      Member mf=ll.getMember("min",par);
-      
-      Class[] par1=new Class[2];
-      par1[0]=Float.TYPE;
-      par1[1]=Float.TYPE;
-      
-      if ((mf!=null) && 
-          (mf.equals(new Method(math.getMethod("min",par1)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Attempt of access to the field PI");
-    try {
-      Class[] par=new Class[0];
-      Member f=ll.getMember("PI",par);
-      if ((f!=null) && (f.getName().equals("PI")))
-	t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Checking assignment of state dependence ");
-    try {
-      ll.markStateDependent("random",null);
-      if (!ll.isStateless(ll.getMember("random",null)))
-	t.testOK();
-      else
-	t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Creating a library with dot operations allowed ");
-    Library ldot=null;
-    try {
-      Class[] stl=new Class[1];
-      Class[] dynl=new Class[1];
-      Class[] dotl=new Class[2];
-      stl[0]=Class.forName("java.lang.Math");
-      dynl[0]=Class.forName("java.util.Random");
-      dotl[0]=Class.forName("java.util.Hashtable");
-      dotl[1]=Class.forName("java.util.Vector");
-      
-      ldot=new Library(stl,dynl,dotl);
-      t.testOK();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Resolving size() on Hashtable");
-    try {
-      Class[] params=new Class[0];
-      Class htable=Class.forName("java.util.Hashtable");
-      Member mf=ldot.getMember(htable,"size",params);
-      if ((mf!=null) && 
-          (mf.equals(new Method(htable.getMethod("size",params)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Resolving size() on Vector");
-    try {
-      Class[] params=new Class[0];
-      Class vctr=Class.forName("java.util.Vector");
-      Member mf=ldot.getMember(vctr,"size",params);
-      if ((mf!=null) && 
-          (mf.equals(new Method(vctr.getMethod("size",params)))))
-	  t.testOK(); else t.testFail();
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-    t.startTest("Ensuring size() is not leaked to global context.");
-    try {
-      Class[] params=new Class[0];
-      try {
-        Member mf=ldot.getMember("size",params);
-        t.testFail();
-      } catch (CompilationException exc) {
-        t.testOK();
-      };
-    } catch (Throwable e) {
-      Debug.reportThrowable(e);
-      t.testFail();
-    };
-
-  };
-
+  //+-+-+-+TESTS  
+//-  // ------------------------- TESTSUITE -------------------------------
+//-  
+//-  /**
+//-   * Performs unitary test of the library.
+//-   * @param args ignored.
+//-   */
+//-  public static void main(String[] args) {
+//-    Tester t=new Tester(System.out);
+//-    test(t);
+//-    t.summarize();
+//-  };
+//-
+//-  /**
+//-   * Performs unitary test of the library.
+//-   * <p> Used if all package is being tested and not just codegen.
+//-   * @param t Tester to report test results.
+//-   */
+//-  public static void test(Tester t) {
+//-    Library ll=null;
+//-    Class math=null;
+//-    try {
+//-      math=Class.forName("java.lang.Math");
+//-    } catch(ClassNotFoundException e) {
+//-      Debug.println("It is IMPOSSIBLE :)");
+//-    };
+//-
+//-    t.startTest("Creating the library of java.lang.Math");
+//-    try {
+//-      Class[] sl=new Class[1];
+//-      sl[0]=math;
+//-      Library l=new Library(sl,null);
+//-      ll=l;
+//-      t.testOK();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Attempt of invocation round(double)");
+//-    try {
+//-      Class[] par=new Class[1];
+//-      par[0]=Double.TYPE;
+//-      Member mf=ll.getMember(null,"round",par);
+//-      if ((mf!=null) && 
+//-          (mf.equals(math.getMethod("round",par))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Attempt of invocation round(float)");
+//-    try {
+//-      Class[] par=new Class[1];
+//-      par[0]=Float.TYPE;
+//-      Member mf=ll.getMember(null,"round",par);
+//-      if ((mf!=null) && 
+//-          (mf.equals(math.getMethod("round",par))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Attempt of invocation round(int) best is round(float)");
+//-    try {
+//-      Class[] par=new Class[1];
+//-      par[0]=Integer.TYPE;
+//-      Member mf=ll.getMember(null,"round",par);
+//-
+//-      Class[] par1=new Class[1];
+//-      par1[0]=Float.TYPE;
+//-
+//-
+//-      if ((mf!=null) && 
+//-          (mf.equals(math.getMethod("round",par1))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Attempt of invocation abs(int) best is abs(int)");
+//-    try {
+//-      Class[] par=new Class[1];
+//-      par[0]=Integer.TYPE;
+//-      Member mf=ll.getMember(null,"abs",par);
+//-
+//-      Class[] par1=new Class[1];
+//-      par1[0]=Integer.TYPE;
+//-
+//-      if ((mf!=null) && 
+//-          (mf.equals(math.getMethod("abs",par1))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Attempt of invocation abs(byte) best is abs(int)");
+//-    try {
+//-      Class[] par=new Class[1];
+//-      par[0]=Byte.TYPE;
+//-      Member mf=ll.getMember(null,"abs",par);
+//-      
+//-      Class[] par1=new Class[1];
+//-      par1[0]=Integer.TYPE;
+//-      
+//-      if ((mf!=null) && 
+//-          (mf.equals(math.getMethod("abs",par1))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Attempt of invocation abs(char) best is abs(int)");
+//-    try {
+//-      Class[] par=new Class[1];
+//-      par[0]=Character.TYPE;
+//-      Member mf=ll.getMember(null,"abs",par);
+//-      
+//-      Class[] par1=new Class[1];
+//-      par1[0]=Integer.TYPE;
+//-      
+//-      if ((mf!=null) && 
+//-          (mf.equals(math.getMethod("abs",par1))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-    
+//-    t.startTest("Attempt of invocation min(int,float) best is min(float,float)");
+//-    try {
+//-      Class[] par=new Class[2];
+//-      par[0]=Integer.TYPE;
+//-      par[1]=Float.TYPE;
+//-      Member mf=ll.getMember(null,"min",par);
+//-      
+//-      Class[] par1=new Class[2];
+//-      par1[0]=Float.TYPE;
+//-      par1[1]=Float.TYPE;
+//-      
+//-      if ((mf!=null) && 
+//-          (mf.equals(math.getMethod("min",par1))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Attempt of access to the field PI");
+//-    try {
+//-      Class[] par=new Class[0];
+//-      Member f=ll.getMember(null,"PI",par);
+//-      if ((f!=null) && (f.getName().equals("PI")))
+//-	t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Checking assignment of state dependence ");
+//-    try {
+//-      ll.markStateDependent("random",null);
+//-      if (!ll.isStateless(ll.getMember(null,"random",null)))
+//-	t.testOK();
+//-      else
+//-	t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Creating a library with dot operations allowed ");
+//-    Library ldot=null;
+//-    try {
+//-      Class[] stl=new Class[1];
+//-      Class[] dynl=new Class[1];
+//-      Class[] dotl=new Class[2];
+//-      stl[0]=Class.forName("java.lang.Math");
+//-      dynl[0]=Class.forName("java.util.Random");
+//-      dotl[0]=Class.forName("java.util.Hashtable");
+//-      dotl[1]=Class.forName("java.util.Vector");
+//-      
+//-      ldot=new Library(stl,dynl,dotl);
+//-      t.testOK();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Resolving size() on Hashtable");
+//-    try {
+//-      Class[] params=new Class[0];
+//-      Class htable=Class.forName("java.util.Hashtable");
+//-      Member mf=ldot.getMember(htable,"size",params);
+//-      if ((mf!=null) && 
+//-          (mf.equals(htable.getMethod("size",params))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Resolving size() on Vector");
+//-    try {
+//-      Class[] params=new Class[0];
+//-      Class vctr=Class.forName("java.util.Vector");
+//-      Member mf=ldot.getMember(vctr,"size",params);
+//-      if ((mf!=null) && 
+//-          (mf.equals(vctr.getMethod("size",params))))
+//-	  t.testOK(); else t.testFail();
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-    t.startTest("Ensuring size() is not leaked to global context.");
+//-    try {
+//-      Class[] params=new Class[0];
+//-      try {
+//-        Member mf=ldot.getMember(null,"size",params);
+//-        t.testFail();
+//-      } catch (CompilationException exc) {
+//-        t.testOK();
+//-      };
+//-    } catch (Throwable e) {
+//-      Debug.reportThrowable(e);
+//-      t.testFail();
+//-    };
+//-
+//-  };
+  //+-+-+-+TESTS
 };
