@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: OPcall.java,v 1.6 2001/02/20 16:01:41 metlov Exp $
+ * $Id: OPcall.java,v 1.7 2001/08/23 14:30:21 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -14,7 +14,9 @@
 
 package gnu.jel;
 
-import gnu.jel.reflect.Member;
+import java.lang.reflect.Member;
+import java.lang.reflect.Field;
+import java.lang.reflect.Method;
 import gnu.jel.debug.Debug;
 import java.util.Stack;
 
@@ -42,7 +44,7 @@ public class OPcall extends OPfunction {
     throws CompilationException {
     this.m=m;
     this.attemptEval=attemptEval;
-    Class[] reqParamTypes=m.getParameterTypes();
+    Class[] reqParamTypes=TypesStack.getParameterTypes(m);
     nplv=reqParamTypes.length;
     
     int thisIdx=0;
@@ -66,7 +68,8 @@ public class OPcall extends OPfunction {
     };
     
     // push & store the result type
-    typesStk.pushID(resID=m.getTypeID(),resType=m.getType());
+    resType=TypesStack.getType(m);
+    typesStk.pushID(resID=TypesStack.typeID(resType),resType);
     //    System.out.println("MAKING CALL TO "+m.getName()+
     //                       ClassFile.getSignature(m)+" returning "+
     //                       m.getType());
@@ -119,7 +122,11 @@ public class OPcall extends OPfunction {
           cOPload=(OPload) cOPload.prev;
         };
       };
-      Object res=m.eval(null,params);
+      Object res;
+      
+      if (m instanceof Method) res=((Method)m).invoke(null,params);
+      else if (m instanceof Field) res=((Field)m).get(null);
+      else return; // nothing else will be evaluated
       
       try { // if there are problems below they should be reported
         if (nParams==0) { // if there are no parameters
@@ -192,7 +199,7 @@ public class OPcall extends OPfunction {
       cf.labels_unblock();
       
       // call the method / get field
-      m.code(cf);
+      cf.codeM(m);
       
       int nParams=nplv;
       while(nParams-->0) cf.typesStk.pop();
