@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: OPload.java,v 1.1 2000/02/11 22:08:18 metlov Exp $
+ * $Id: OPload.java,v 1.2 2000/09/09 19:51:48 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -21,20 +21,20 @@ public class OPload extends OP {
   
   public OPload(TypesStack typesStk, Class resType, Object what) {
     this.what=what;
-    this.resID=TypesStack.primitiveID(resType);
+    this.resID=TypesStack.typeID(resType);
     this.resType=resType;
     if (typesStk!=null) typesStk.pushID(resID,resType);
   };
 
   public OPload(TypesStack typesStk, Object what) {
     this.what=what;
-    this.resID=TypesStack.primitiveID(what);
+    this.resID=TypesStack.typeIDObject(what);
 
     if (Debug.enabled)
       Debug.assert((resID!=8) || (what instanceof String));
     
     if (resID!=8)
-      this.resType=TypesStack.primitiveTypes[resID];
+      this.resType=TypesStack.specialTypes[resID];
     else
       this.resType=what.getClass();
 
