@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: TypesStack.java,v 1.6 2001/01/22 15:58:20 metlov Exp $
+ * $Id: TypesStack.java,v 1.8 2001/08/23 14:30:21 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -15,8 +15,15 @@
 package gnu.jel;
 
 import gnu.jel.debug.Debug;
-import gnu.jel.debug.Tester;
 import java.util.Stack;
+import java.lang.reflect.Member;
+import java.lang.reflect.Field;
+import java.lang.reflect.Method;
+import java.lang.reflect.Constructor;
+
+//+-+-+-+TESTS
+//-import gnu.jel.debug.Tester;
+//+-+-+-+TESTS
 
 /**
  * Organizes stack of types, supports type identification and
@@ -209,7 +216,8 @@ public class TypesStack implements Cloneable {
    * Pushes the class with given typeID into stack.
    */
   public final void pushID(int id,Class c) {
-    Debug.assert(! ((id==8) && (c==null)));
+    if (Debug.enabled)
+      Debug.assert(! ((id==8) && (c==null)));
     typeIDs.push(id);
     if (c!=null) 
       classes.push(c); 
@@ -393,115 +401,243 @@ public class TypesStack implements Cloneable {
     };
   };
 
-  
-  //================================================================
-  //======================== UNITARY TESTS =========================
-  //================================================================
-
   /**
-   * Performs unitary test of this class.
-   * @param args ignored.
+   * Used to get return type of a class member.
+   * <P>The type of a Method is its return type, the type of a Constructor is
+   * void.
+   * @param  m member whose type is to be determined
+   * @return type of the member
    */
-  public static void main(String[] args) {
-    if (Debug.enabled) {
-      Tester t=new Tester(System.out);
-      test(t);
-      t.summarize();
-    };
+  public static Class getType(Member m) {
+    if (m instanceof Method) return ((Method)m).getReturnType();
+    if (m instanceof Field) return ((Field)m).getType();
+    if (m instanceof LocalField) return ((LocalField)m).getType();
+    // otherwise it must be java.lang.reflect.Constructor
+    if (Debug.enabled)
+      Debug.assert(m instanceof java.lang.reflect.Constructor);
+    return specialTypes[9]; // java.lang.reflect.Void.TYPE
   };
 
   /**
-   * Performs unitary test of this class.
-   * <p> Used if all package is being tested and not just codegen.
-   * @param t Tester to report test results.
+   * Used to get types of formal parameters of a member.
+   * <P> The reference to the class instance "this" is not counted by
+   * this method.
+   * @param  m member whose formal parameters are to be obtained
+   * @return array of formal parameter types (empty array if none).
    */
-  public static void test(Tester t) {
-    if (Debug.enabled) {
-      TypesStack ts=null;
-      try {
-        t.startTest("ID(Boolean.TYPE)==ID(new Boolean(true))");
-        t.compare(typeID(Boolean.TYPE),typeIDObject(new Boolean(true)));
-        t.startTest("ID(Byte.TYPE)==ID(new Byte(0))");
-        t.compare(typeID(Byte.TYPE),typeIDObject(new Byte((byte)0)));
-        t.startTest("ID(\"string\".getClass())==11");
-        t.compare(typeID(("string").getClass()),11);
-        t.startTest("ID((new Object()).getClass())==8");
-        t.compare(typeID((new Object()).getClass()),8);
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
-      
-      try {
-        t.startTest("Make a new types stack");
-        ts=new TypesStack();
-        t.testOK();
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
+  public static Class[] getParameterTypes(Member m) {
+    if (m instanceof Method) return ((Method)m).getParameterTypes();
+    if (m instanceof LocalMethod) return ((LocalMethod)m).getParameterTypes();
+    if (m instanceof Constructor) return ((Constructor)m).getParameterTypes();
 
-      try {
-        t.startTest("push(Integer.TYPE);pushID(4); pop()==pop()");
-        ts.push(Integer.TYPE);
-        ts.pushID(4);
-        t.compare(ts.pop(),ts.pop());
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
-
-      try {
-        t.startTest("push(Integer.TYPE,0);pushID(4,0); pop()==pop()");
-        ts.push(Integer.TYPE,0);
-        ts.pushID(4,0);
-        t.compare(ts.pop(),ts.pop());
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
-
-      try {
-        t.startTest("push(Long.TYPE);pushID(6);pushID(4,1);"+
-                    "peekID()==peekID(1)+2");
-        ts.push(Long.TYPE);
-        ts.pushID(6);
-        ts.pushID(4,1);
-        t.compare(ts.peekID(),ts.peekID(1)+2);
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
-
-      try {
-        t.startTest("pop();peekID()==peekID(1)-1");
-        ts.pop();
-        t.compare(ts.peekID(),ts.peekID(1)-1);
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
-
-      try {
-        t.startTest("getMaxOccupation()==4");
-        t.compare(ts.getMaxOccupation(),4);
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
-
-      try {
-        t.startTest("pop();pop(); size()==0");
-        ts.pop();
-        ts.pop();
-        t.compare(ts.size(),0);
-      } catch (Throwable exc) {
-        Debug.reportThrowable(exc);
-        t.testFail();
-      };
-
-
-    };
+    if (Debug.enabled)
+      Debug.assert((m instanceof Field)||(m instanceof LocalField));
+    
+    return new Class[0];
   };
 
+  /**
+   * Computes signature of the given member.
+   * @param m the member to compute the sugnature of.
+   * @return the signature.
+   */
+  public static String getSignature(Member m) {
+    StringBuffer signature=new StringBuffer();
+    if (!isField(m)) {
+      Class parameters[]=getParameterTypes(m);
+      signature.append('(');
+      for(int i=0;i<parameters.length;i++) 
+        appendSignature(signature,parameters[i]);
+      signature.append(')');
+    };
+    appendSignature(signature,getType(m));
+    return signature.toString();
+  };
+
+  public static boolean isField(Member m) {
+    return (m instanceof Field) || ((m instanceof LocalField)
+                                    && !(m instanceof LocalMethod));
+  };
+
+  /**
+   * Computes the signature of the given class.
+   * <P> The signature of the class (Field descriptor) is the string and 
+   * it's format is described in the paragraph 4.3.2 of the Java VM 
+   * specification (ISBN 0-201-63451-1).
+   * <P>The same can be done using <TT>java.lang.Class.getName()</TT> by 
+   * converting it's result into the "historical form".
+   * <P> This utility method can be used outside of the JEL package
+   * it does not involve any JEL specific assumptions and should follow
+   * JVM Specification precisely.
+   * @param cls is the class to compute the signature of. Can be primitive or
+   *            array type.
+   * @return the class signature.
+   */
+  public static String getSignature(Class cls) {
+    return appendSignature(new StringBuffer(),cls).toString();
+  };
+
+  private static StringBuffer appendSignature(StringBuffer buff, Class cls) {
+    if (cls.isPrimitive()) 
+      buff.append(TypesStack.primitiveCodes[TypesStack.baseType[TypesStack.typeID(cls)]]);
+    else if (cls.isArray()) {
+      buff.append('[');
+      appendSignature(buff,cls.getComponentType());
+    } else { // just a class
+      buff.append('L');
+      appendHistoricalForm(buff,cls.getName());
+      buff.append(';');
+    };
+    return buff;
+  };
+
+  public static String toHistoricalForm(String className) {
+    return appendHistoricalForm(new StringBuffer(),className).toString();
+  };
+
+  private static StringBuffer appendHistoricalForm(StringBuffer buff,
+                                                   String className) {
+    int namelen=className.length();
+    for(int i=0;i<className.length();i++) {
+      char cch=className.charAt(i);
+      if (cch=='.') cch='/';
+      buff.append(cch);
+    };
+    return buff;
+  };
+
+
+  //+-+-+-+TESTS  
+//-  //================================================================
+//-  //======================== UNITARY TESTS =========================
+//-  //================================================================
+//-
+//-  /**
+//-   * Performs unitary test of this class.
+//-   * @param args ignored.
+//-   */
+//-  public static void main(String[] args) {
+//-    if (Debug.enabled) {
+//-      Tester t=new Tester(System.out);
+//-      test(t);
+//-      t.summarize();
+//-    };
+//-  };
+//-
+//-  /**
+//-   * Performs unitary test of this class.
+//-   * <p> Used if all package is being tested and not just codegen.
+//-   * @param t Tester to report test results.
+//-   */
+//-  public static void test(Tester t) {
+//-    if (Debug.enabled) {
+//-      // Signature computing tests
+//-      {
+//-        t.startTest("toHistoricalForm(\"java.lang.String\")");
+//-        t.compare(toHistoricalForm("java.lang.String"),
+//-                  "java/lang/String");
+//-
+//-        t.startTest("getSignature((\"a string\").getClass())");
+//-        t.compare(getSignature(("a string").getClass()),
+//-                  "Ljava/lang/String;");
+//-	
+//-        t.startTest("getSignature((new int[10]).getClass())");
+//-        t.compare(getSignature((new int[10]).getClass()),
+//-                  "[I");
+//-	
+//-        t.startTest("getSignature((new Object[10]).getClass())");
+//-        t.compare(getSignature((new Object[10]).getClass()),
+//-                  "[Ljava/lang/Object;");
+//-
+//-      };
+//-
+//-
+//-      TypesStack ts=null;
+//-      try {
+//-        t.startTest("ID(Boolean.TYPE)==ID(new Boolean(true))");
+//-        t.compare(typeID(Boolean.TYPE),typeIDObject(new Boolean(true)));
+//-        t.startTest("ID(Byte.TYPE)==ID(new Byte(0))");
+//-        t.compare(typeID(Byte.TYPE),typeIDObject(new Byte((byte)0)));
+//-        t.startTest("ID(\"string\".getClass())==11");
+//-        t.compare(typeID(("string").getClass()),11);
+//-        t.startTest("ID((new Object()).getClass())==8");
+//-        t.compare(typeID((new Object()).getClass()),8);
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-      
+//-      try {
+//-        t.startTest("Make a new types stack");
+//-        ts=new TypesStack();
+//-        t.testOK();
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-
+//-      try {
+//-        t.startTest("push(Integer.TYPE);pushID(4); pop()==pop()");
+//-        ts.push(Integer.TYPE);
+//-        ts.pushID(4);
+//-        t.compare(ts.pop(),ts.pop());
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-
+//-      try {
+//-        t.startTest("push(Integer.TYPE,0);pushID(4,0); pop()==pop()");
+//-        ts.push(Integer.TYPE,0);
+//-        ts.pushID(4,0);
+//-        t.compare(ts.pop(),ts.pop());
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-
+//-      try {
+//-        t.startTest("push(Long.TYPE);pushID(6);pushID(4,1);"+
+//-                    "peekID()==peekID(1)+2");
+//-        ts.push(Long.TYPE);
+//-        ts.pushID(6);
+//-        ts.pushID(4,1);
+//-        t.compare(ts.peekID(),ts.peekID(1)+2);
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-
+//-      try {
+//-        t.startTest("pop();peekID()==peekID(1)-1");
+//-        ts.pop();
+//-        t.compare(ts.peekID(),ts.peekID(1)-1);
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-
+//-      try {
+//-        t.startTest("getMaxOccupation()==4");
+//-        t.compare(ts.getMaxOccupation(),4);
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-
+//-      try {
+//-        t.startTest("pop();pop(); size()==0");
+//-        ts.pop();
+//-        ts.pop();
+//-        t.compare(ts.size(),0);
+//-      } catch (Throwable exc) {
+//-        Debug.reportThrowable(exc);
+//-        t.testFail();
+//-      };
+//-
+//-
+//-    };
+//-  };
+  //+-+-+-+TESTS
+
 };
